import { RabbitSubscribe } from '@golevelup/nestjs-rabbitmq';
import { Controller, Inject } from '@nestjs/common';
import { WINSTON_MODULE_PROVIDER } from 'nest-winston';
import { Logger } from 'winston';
import { level } from './logger/level.enum';
import loggerOptions from './logger/logger';

@Controller()
export class ConfigurationController {
  constructor(
    @Inject(WINSTON_MODULE_PROVIDER) private readonly logger: Logger,
  ) {}

  @RabbitSubscribe({
    name: 'loggingConfigurationCreateInfo',
  })
  public async configurationCreatedInfoHandler(message: string) {
    this.logger.configure(
      loggerOptions('configuration', level.info, 'info', 'create'),
    );
    this.logger.info(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: 'loggingConfigurationCreateWarning',
  })
  public async configurationCreatedWarningHandler(message: string) {
    this.logger.configure(
      loggerOptions('configuration', level.warning, 'warning', 'create'),
    );
    this.logger.warning(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: 'loggingConfigurationCreateCrit',
  })
  public async configurationCreatedCriticalHandler(message: string) {
    this.logger.configure(
      loggerOptions('configuration', level.crit, 'critical', 'create'),
    );
    this.logger.crit(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: 'loggingConfigurationDeleteInfo',
  })
  public async configurationDeletedInfoHandler(message: string) {
    this.logger.configure(
      loggerOptions('configuration', level.info, 'info', 'delete'),
    );
    this.logger.info(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: 'loggingConfigurationDeleteCrit',
  })
  public async configurationDeletedCriticalHandler(message: string) {
    this.logger.configure(
      loggerOptions('configuration', level.crit, 'critical', 'delete'),
    );
    this.logger.crit(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: 'loggingConfigurationReadWarning',
  })
  public async configurationReadWarningHandler(message: string) {
    this.logger.configure(
      loggerOptions('configuration', level.warning, 'warning', 'read'),
    );
    this.logger.warning(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: 'loggingConfigurationUpdateInfo',
  })
  public async configurationUpdatedInfoHandler(message: string) {
    this.logger.configure(
      loggerOptions('configuration', level.info, 'info', 'update'),
    );
    this.logger.info(JSON.parse(message));
  }

  @RabbitSubscribe({
    name: 'loggingConfigurationUpdateCrit',
  })
  public async configurationUpdatedCriticalHandler(message: string) {
    this.logger.configure(
      loggerOptions('configuration', level.crit, 'critical', 'update'),
    );
    this.logger.crit(JSON.parse(message));
  }
}
